from __future__ import print_function
import argparse
import torch.utils.data
from torchvision import datasets, transforms
from torch.autograd import Variable
import os
import numpy as np
from Fca_module import FcaNet34
parser = argparse.ArgumentParser(description='PyTorch MNIST Example')
parser.add_argument('--batch-size', type=int, default=64, metavar='N',
                    help='input batch size for training (default: 64)')
parser.add_argument('--test-batch-size', type=int, default=1000, metavar='N',
                    help='input batch size for testing (default: 1000)')
parser.add_argument('--epochs', type=int, default=10, metavar='N',
                    help='number of epochs to train (default: 10)')
parser.add_argument('--lr', type=float, default=0.01, metavar='LR',
                    help='learning rate (default: 0.01)')
parser.add_argument('--momentum', type=float, default=0.5, metavar='M',
                    help='SGD momentum (default: 0.5)')
parser.add_argument('--no-cuda', action='store_true', default=False,
                    help='disables CUDA training')
parser.add_argument('--seed', type=int, default=1, metavar='S',
                    help='random seed (default: 1)')
parser.add_argument('--log-interval', type=int, default=10, metavar='N',
                    help='how many batches to wait before logging training status')
parser.add_argument('--save_dir', type=str, default='./train',
                    help="the path to save the trained model")
args = parser.parse_args()
args.cuda = not args.no_cuda and torch.cuda.is_available()
if not os.path.exists(args.save_dir):
    os.makedirs(args.save_dir)
torch.manual_seed(args.seed)
if args.cuda:
    torch.cuda.manual_seed(args.seed)
data_transform = {
        "train": transforms.Compose([transforms.RandomResizedCrop(224), transforms.RandomHorizontalFlip(),
                                     transforms.ToTensor(),
                                     transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]),
        "val": transforms.Compose([transforms.Resize(256), transforms.CenterCrop(224), transforms.ToTensor(),
                                   transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])])}
data_root = os.path.abspath(os.path.join(os.getcwd(), "./.."))
image_path = data_root + "/det_set/data/"
assert os.path.exists(image_path), "{} path does not exist.".format(image_path)
batch_size = 40
train_dataset = datasets.ImageFolder(root=os.path.join(image_path, "train"), transform=data_transform["train"])
train_num = len(train_dataset)
train_loader = torch.utils.data.DataLoader(train_dataset, batch_size=batch_size, shuffle=True, num_workers=0)
net = FcaNet34(num_classes=10)
net.to('cuda')
model_path = os.path.join('fca34-99.3.pth')
net.load_state_dict(torch.load(model_path))


def generate_feature():
    net.eval()
    cnt = 0
    out_target = []
    out_data = []
    out_output = []
    for data, target in train_loader:
        cnt += len(data)
        print("process...: %d/%d" % (cnt, len(train_loader)))
        if args.cuda:
            data, target = data.cuda(), target.cuda()
        data, target = Variable(data, volatile=True), Variable(target)
        output = net(data)
        output_np = output.data.cpu().numpy()
        target_np = target.data.cpu().numpy()
        data_np = data.data.cpu().numpy()
        out_output.append(output_np)
        out_target.append(target_np[:, np.newaxis])
        out_data.append(np.squeeze(data_np))
    output_array = np.concatenate(out_output, axis=0)
    target_array = np.concatenate(out_target, axis=0)
    data_array = np.concatenate(out_data, axis=0)
    np.save(os.path.join(args.save_dir, 'fca_output.npy'), output_array, allow_pickle=False)
    np.save(os.path.join(args.save_dir, 'fca_target.npy'), target_array, allow_pickle=False)
    np.save(os.path.join(args.save_dir, 'fca_data.npy'), data_array, allow_pickle=False)


generate_feature()
